/********************************************************************/
/*                                                                  */
/* INSTALL programm for LYNX/2 2.8.5                                */
/*                                                                  */
/* Author: Michel SUCH. Email: msuch@free.fr                        */
/*                                                                  */
/********************************************************************/

   call init_prog
   call get_langs
   call build_dirs
   call build_starter
   call update_configsys
   call update_desktop
   call inst_srd2
   say lynx "installed successfully!"
   say "You can now start lynx from the desktop"
   say "or, after rebooting, type RUNLYNX on a command line."
exit /* main */


init_prog:

   if RxfuncQuery('sysloadfuncs') <> 0 then do
      Call RxFuncAdd 'SysLoadFuncs','RexxUtil','SysLoadFuncs'
      Call SysLoadFuncs
   end
   lynx = "LYNX/2 2.8.5"

   parse source . . pgm .
   lxdrv = filespec('d', pgm)
   lxdir = filespec('p', pgm)
   basedir = lxdrv || lxdir
   dir = directory( delstr(basedir, length(basedir)))
   unixdir = translate(basedir, '/', '\')
   lynxhelp = "file://localhost/"unixdir"lib/lynx_help/lynx_help_main.html.gz"
   starter = basedir || "bin\RUNLYNX.CMD"

   say "Lynx/2 installation program"
   say ""
return /* init_prog */


get_langs:
   if translate(lxdir) <> "\LYNX2-8-5\" then do
      say "since the LYNX2-8-5 directory is not at the root level,"
      say "no NLS support will be provided."
      say "Language will be set to English."
      lang = "en"
      return
   end

   langs = ''
   linguas = ''
   call sysfiletree "share\locale\lynx.mo", lst., "so"
   if lst.0 = 0 then do
      say "No NLS support found,"
      say "language will be set to english."
      lang = "EN"
      return
   end

   say "Language selection"
   say
   do i = 1 to lst.0
      parse upper var lst.i ."\SHARE\LOCALE\"lang"\LC_MESSAGES\".
      select
         when lang = "CS" then lingua = "Czech"
         when lang = "DA" then lingua = "Dansk"
         when lang = "DE" then lingua = "German"
         when lang = "FR" then lingua = "French"
         when lang = "JA" then lingua = "Japanese"
         when lang = "NL" then lingua = "Dutch"
         when lang = "PT_BR" then lingua = "Brasilian"
         when lang = "RU" then lingua = "Russian"
         when lang = "SL" then lingua = "Slovenian"
         when lang = "SV" then lingua = "Swedish"
         otherwise iterate
      end /* select */
      say lang":" lingua
      langs = langs lang
      linguas = linguas lingua
   end /* while */

   Say "Choose a language in above list or press enter for english"
   parse pull lang
   do while wordpos(translate(lang), langs) = 0 & lang <> ''
      say "Invalid language:" lang
      parse pull lang
   end
   lang = translate(lang)

   if lang = '' then do
      lang = 'en'
      lingua = 'English'
   end
   else do
   lingua = word(linguas, wordpos(lang, langs))
   end
   say "Language will be set to" lingua
return /* get_langs */


build_dirs:

   say "Creating home, incoming and tmp subdirectories..."
   call sysmkdir "home"
   call sysmkdir "tmp"
   call sysmkdir "incoming"

   if stream("home\lynx.cfg", 'c', 'query exist') = '' then do
      "@copy lib\lynx.cfg home\lynx.cfg >nul 2>nul"
      call lineout "home\lynx.cfg", "helpfile:"lynxhelp
      call lineout "home\lynx.cfg"
   end

   if stream("home\lynx.lss", 'c', 'query exist') = '' then do
      "@copy lib\lynx.lss home\lynx.lss >nul 2>nul"
   end

   if stream("home\.mime.types", 'c', 'query exist') = '' then do
      "@copy lib\.mime.types home\.mime.types >nul 2>nul"
   end
   else do
      say "Updating file home\.mime.types"
      call merge_mime "lib\.mime.types home\.mime.types"
   end

   if stream("home\.mailcap", 'c', 'query exist') = '' then do
      "@copy lib\.mailcap home\.mailcap >nul 2>nul"
   end
   else do
      say "Updating file home\.mailcap"
      call merge_cap "lib\.mailcap home\.mailcap"
   end

   if stream(basedir || "home\.wgetrc", 'c', 'query exist') = '' then do
      "@copy" basedir"lib\wgetrc home\.wgetrc >nul 2>nul"
   end
return /* build_dirs */


build_starter:

   say "Creating startup program runlynx.cmd..."
   call sysfiledelete starter

   call lineout starter, "/************************************************************/"
   call lineout starter, "/* LYNX startup script                                      */"
   call lineout starter, "/************************************************************/"

   call lineout starter, "  parse source . . pgm ."
   call lineout starter, "  exepath = filespec('d', pgm) || filespec('p', pgm)"
   call lineout starter, "  curdir = directory()"
   call lineout starter, "  newdir = directory(delstr(exepath, length(exepath)))"
   call lineout starter, "  x = lastpos('\', delstr(exepath, length(exepath) - 1))"
   call lineout starter, "  root = translate( delstr( exepath, x + 1), '/', '\')"
   call lineout starter, "  '@setlocal'"
   call lineout starter, "  '@set shell=%os2_shell%'"
   call lineout starter, "  '@set lang="lang"'"
   call lineout starter, "  '@set home='root'home'"
   call lineout starter, "  '@set terminfo='root'share/terminfo'"
   call lineout starter, "  '@SET TERM=ansi'"
   call lineout starter, "  '@set tmpdir='root'tmp'"
   call lineout starter, "  '@set lynx_cfg='root'home/lynx.cfg'"
   call lineout starter, "  '@set lynx_lss='root'home/lynx.lss'"
   call lineout starter, "  '@'exepath'lynx.exe %1 %2 %3 %4 %5 %6 %7 %8 %9'"
   call lineout starter, "  if rc <> 0 then '@pause'"
   call lineout starter, "  '@endlocal'"
   call lineout starter, "  curdir = directory(curdir)"
   call lineout starter, "exit /* RUNLYNX.CMD */"
   call lineout starter
   return /* build_starter */


update_configsys:
procedure expose basedir

   lxpath = translate(basedir)"BIN;"
   /* get boot drive */
   sysdrv = left( value( 'system_ini',, "os2environment"), 3)
   if sysdrv = '' then return /* should not happen! */
   config = sysdrv || "config.sys"
   i = 0
   do while lines(config) > 0
      i = i + 1
      ln.i = linein( config)
      ln = translate(ln.i)
      if pos( "SET PATH=", ln) = 1 then do
         if pos( lxpath, ln) > 0 then do
            call stream config, 'c', 'close'
            return
         end
         parse var ln.i set"="val
         ln.i = set"="lxpath || val
         say ln.i
      end
   end
   call stream config, 'c', 'close'
   confback = delstr( config, length(config)-2) || "lnx"
   say "Saving" config "to" confback
   "@copy" config confback ">nul 2>nul"
   say "Updating file" config
   call sysfiledelete config
   do j = 1 to i
      call lineout config, ln.j
   end
   call lineout config
return /* update_configsys */


update_desktop:

   say "Creating lynx/2 2.8.5 folder and objects on desktop..."

   call sysdestroyobject "<LYNX_FOLDER>"
   Call SysCreateObject,
         "WPFolder",,
         "LYNX/2 2.8.5",,
         "<WP_DESKTOP>",,
         "OBJECTID=<LYNX_FOLDER>;"||,
         "ICONFILE="||basedir||"lib\html.ico;",,
         "REPLACE"
   if result <> 1 then call error "Cannot create LYNX/2 folder"

   call sysdestroyobject "<WGET_CFG>"
   call SysCreateObject,
      'WPProgram',,
      'EDIT WGET CONFIGURATION FILE',,
      '<LYNX_FOLDER>',,
      'EXENAME=E.EXE;'||,
      ';OBJECTID=<WGET_CFG>;'||,
      ';PARAMETERS='basedir'home\.wgetrc;',,
                        'replace'
   if result <> 1 then call error "Cannot create EDIT WGET CONFIGURATION FILE object"

   call sysdestroyobject "<LYNX_CFG>"
   call SysCreateObject,
      'WPProgram',,
      'EDIT LYNX CONFIGURATION FILE',,
      '<LYNX_FOLDER>',,
      'EXENAME=E.EXE;'||,
      ';OBJECTID=<LYNX_CFG>;'||,
      ';PARAMETERS='basedir'home\lynx.cfg;',,
                        'replace'
   if result <> 1 then call error "Cannot create EDIT LYNX CONFIGURATION FILE object"

   exename = basedir"bin\runlynx.cmd"
   call sysdestroyobject "<LYNX_HLP>"
   call SysCreateObject,
      'WPProgram',,
      'LYNX/2 HELP',,
      '<LYNX_FOLDER>',,
      'EXENAME='exename';'||,
      ';OBJECTID=<LYNX_HLP>;'||,
      ";ICONFILE="||basedir||"lib\lynx.ico;"||,
      ';PARAMETERS='lynxhelp';'||,
                        'replace'
   if result <> 1 then call error "Cannot create LYNX HELP OBJECT"

   call sysdestroyobject "<LYNX_PGM>"
   call SysCreateObject,
      'WPProgram',,
      'LYNX/2 BROWSER',,
         "<LYNX_FOLDER>",,
      'EXENAME='exename';'||,
      ';OBJECTID=<LYNX_PGM>;'||,
      ";ICONFILE="||basedir||"lib\lynx.ico;"||,
                        'replace'
   if result <> 1 then call error "Cannot create LYNX/2 object"
return /* update_desktop */


merge_cap:
procedure

   parse arg fnin fnout
   do while lines(fnin) <> 0
      ln = linein(fnin)
      ln = strip(ln)
      if ln = '' then iterate
      if substr(ln, 1, 1) = "#" then iterate
      keyin = word(ln, 1)
      parse var keyin keyin';'.

      found = 0
      do while lines(fnout) <> 0
         ln2 = linein(fnout);
         ln2 = strip(ln2)
         if ln2 = '' then iterate
         if substr(ln2, 1, 1) = "#" then iterate
         keyout = word(ln2, 1)
         parse var keyout keyout';'.
         if translate(keyin) = translate(keyout) then do
            found = 1
            leave
         end
      end
      call stream fnout, 'c', 'close'
      if found = 0 then do
         call lineout fnout, ln
         call lineout fnout
      end
   end
   call stream fnin, 'c', 'close'
return /* merge_cap */


merge_mime:
procedure

   parse arg fnin fnout
   i = 0
   changed = 0
   do while lines(fnout) <> 0
      i = i + 1
      out.i = linein(fnout)
   end
   call stream fnout, 'c', 'close'
   out.0 = i

   do while lines(fnin) <> 0
      ln = linein(fnin)
      ln = strip(ln)
      if ln = '' then iterate
      if substr(ln, 1, 1) = "#" then iterate
      keyin = word(ln, 1)

      found = 0
      do i = 1 to out.0
         ln2 = out.i
         ln2 = strip(ln2)
         if ln2 = '' then iterate
         if substr(ln2, 1, 1) = "#" then iterate
         keyout = word(ln2, 1)
         if translate(keyin) = translate(keyout) then do
            do j = 2 to words(ln)
               ext = word(ln, j)
               if wordpos(translate(ext), translate(ln2)) = 0 then do
                  ln2 = sortwords(ext, ln2)
                  changed = 1
               end
            end
            if changed = 1 then out.i = ln2
            found = 1
            leave
         end
      end
      if found = 0 then do
         i = out.0 + 1
         out.i = ln
         out.0 = i
         changed = 1
      end
   end
   call stream fnin, 'c', 'close'

   if changed = 1 then do
      call sysfiledelete fnout
      do i = 1 to out.0
         call lineout fnout, out.i
      end
      call lineout fnout
   end
return /* merge_mime */


sortwords:
procedure

   new = arg(1)
   ln = arg(2)
   inserted = 0
   str = word(ln, 1)
   do i = 2 to words(ln)
      w = word(ln, i)
      if inserted = 0 then do
         if new < w then do
            str = str new
            inserted = 1
         end
      end
      str = str w
   end
   if inserted = 0 then str = str new
return str /* sortwords */

inst_srd2:

   srddir = value('srdprofiles', , 'os2environment')
   if srddir = '' then return

   answer = ''
   do while pos(answer, 'YN') = 0
      say "Do you want to install Screen reader/2 profile for Lynx (Y/N),"
      answer = sysgetkey('noecho')
      say answer
      answer = translate(answer)
   end
   if answer = 'N' then return
   dir = directory(srddir)
   if stream("lynx.kpd", 'c', 'query exist') <> '' then do
      say "Saving lynx.kpd as lynx.bak"
      "@copy lynx.kpd lynx.bak >nul 2>nul"
      lynxkpd_found = 1
   end
   "@unzip -o" basedir"bin\lynx_sr2"
   if rc <> 0 then call error "Error unziping file" basedir"bin\lynx_sr2.zip"
   "@call pal lynx"
   if rc <> 0 then do
      if lynxkpd_found = 1 then do
         say "Restoring lynx.bak to lynx.kpd"
         "@copy lynx.bak lynx.kpd >nul 2>nul"
      end
      call error "Error compiling profile lynx.kpd"
   end
   call update_myinit "lynx lynx.exe"
   if base_modified = 1 then do
     "@call pal srd2"
     if rc <> 0 then do
        say "Restoring myinit.bak to myinit.kpd"
        "@copy myinit.bak myinit.kpd >nul 2>nul"
        call error "Error compiling base profile srd2.kpd"
     end
     "@call restart"
   end
return /* inst_srd2 */


update_myinit:

   parse arg profile program
   if already_in_init() = 0 then do
      base_modified = 1
      say "saving myinit.kpd to myinit.bak"
      "@copy myinit.kpd myinit.bak >nul 2>nul"
      call lineout myinit, "initialize '"program"'"
      call lineout myinit, "   push ('"profile"');"
      call stream myinit, 'c', 'close'
   end
return


already_in_init:

   found = 0
   program = translate(program)
   myinit = "myinit.kpd"
   do while lines(myinit) <> 0
      line = strip(translate(space(linein(myinit))))
      if pos("INITIALIZE '"program"'", line) = 1 then
      do
         found = 1
         leave
      end
   end
   call stream myinit, 'c', 'close'
   if found = 0 then
   do
      init = "init.kpd"
      do while lines(init) <> 0
         line = strip(translate(space(linein(init))))
         if pos("INITIALIZE '"program"'", line) = 1 then
         do
            found = 1
            leave
         end
      end
   end
   call stream init, 'c', 'close'
return found


error:

   parse arg msg
   say msg
   call beep 440, 200
exit /* error */
